package au.com.nbnco.framework;

import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.Date;
import java.util.Iterator;
import java.sql.Timestamp;
import java.net.InetAddress;
import java.net.UnknownHostException;

import org.apache.camel.Exchange;


/**
 * Handles the occurrence of an exception
 */

public class ExceptionHandler {

	public Exception addExceptionRecord(Exchange exchange) throws UnknownHostException {
		Exception excp = new Exception();
		Utils utils = new Utils();

		Map headers = exchange.getIn().getHeaders();
		
		excp.setExchangeId(exchange.getExchangeId());
		
		//Look for custom business correlation id set by the application
		//If not,then use the correlation id from the Exchange	
		String correlationId = (null != headers.get("CORRELATION_ID")) ? headers.get("CORRELATION_ID").toString() :
			exchange.getProperty(exchange.CORRELATION_ID, java.lang.String.class);
		excp.setCorrelationId(correlationId);
		
		excp.setContextId(exchange.getContext().getName());
		excp.setRouteId(exchange.getFromRouteId());
		
		//set current timestamp
		Date date= new Date();
		excp.setOccurred(new Timestamp(date.getTime()));
		
		//Get hostname of the server
		excp.setHostName(InetAddress.getLocalHost().getHostName());		
		
		//There can be two kinds of exceptions
		//1. Which has been captured by the DefaultErrorHandler (available as exchange.getException)
		//2. An exception that has been caught by an error handler like a DLC (available as Exchange.EXCEPTION_CAUGHT)
		//Refer to http://camel.apache.org/why-is-the-exception-null-when-i-use-onexception.html
		
		java.lang.Exception excpCaught = (null != exchange.getException()) ? exchange.getException() : 
			exchange.getProperty(Exchange.EXCEPTION_CAUGHT, java.lang.Exception.class);
		
		excp.setMessage(excpCaught.getMessage());
		excp.setMessageHistory(exchange.getProperty(Exchange.MESSAGE_HISTORY, java.lang.String.class));
		//Call a utility function to convert stacktrace to string
		excp.setStackTrace(utils.stackTracetoString(excpCaught));
		
		//Check headers for message retry and set to null if not present
		String retry = (null != headers.get("RETRY")) ? headers.get("RETRY").toString() : null;
		excp.setRetry(retry);	
		
		String retryDestination = (null != headers.get("RETRY_DESTINATION")) ? headers.get("RETRY_DESTINATION").toString() : null;
		excp.setRetryDestination(retryDestination);		
		
		if ((null != headers.get("MAX_RETRY_COUNT")) && (headers.get("MAX_RETRY_COUNT") instanceof Integer)){
			excp.setMaxRetryCount((Integer) headers.get("MAX_RETRY_COUNT"));
		}
		
		
		//Check for empty message payloads
		String payload = (null != exchange.getIn().getBody()) ? exchange.getIn().getBody().toString() : null;
		excp.setPayload(payload);

		
		//iterate through Exchange Headers,provided they are not null
		if (null != headers){
			
			List<Parameter> parameters = new ArrayList<Parameter>();
			Iterator<Map.Entry<String, Object>> iterator = headers.entrySet().iterator();
			
			while (iterator.hasNext()){
				Map.Entry<String, Object> header = iterator.next();
				if (header.getValue() instanceof java.lang.String)
					parameters.add(new Parameter(header.getKey(), header.getValue().toString()));
				}	
			
			excp.setParameters(parameters);

		}
		
		return excp;
	}	
}
