package au.com.nbnco.framework;

import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.Date;
import java.util.Iterator;
import java.sql.Timestamp;
import java.net.InetAddress;
import java.net.UnknownHostException;

import org.apache.camel.Exchange;

/**
 * Handles the occurrence of an audit log.
 */

public class LogHandler {
	
	public Log addLogRecord(Exchange exchange) throws UnknownHostException {
		Log log = new Log();
		
		//Store the headers present in the Camel Exchange
		Map headers = exchange.getIn().getHeaders();
				
		log.setExchangeId(exchange.getExchangeId());
		
		//Look for custom business correlation id set by the application
		//If not,then use the correlation id from the Exchange	
		String correlationId = (null != headers.get("CORRELATION_ID")) ? headers.get("CORRELATION_ID").toString() :
			exchange.getProperty(exchange.CORRELATION_ID, java.lang.String.class);
		log.setCorrelationId(correlationId);
		
		log.setContextId(exchange.getContext().getName());
		log.setRouteId(exchange.getFromRouteId());
		
		//set current timestamp
		Date date= new Date();
		log.setOccurred(new Timestamp(date.getTime()));
		
		//Get hostname of the server
		log.setHostName(InetAddress.getLocalHost().getHostName());
		
		//Check headers for log message and set to null if not present
		String logMessage = (null != headers.get("LOG_MESSAGE")) ? headers.get("LOG_MESSAGE").toString() : null;
		log.setMessage(logMessage);

		//Check for empty message payloads
		String payload = (null != exchange.getIn().getBody()) ? exchange.getIn().getBody().toString() : null;
		log.setPayload(payload);

		
		//iterate through Exchange Headers,provided they are not null
		if (null != headers){
			List<Parameter> parameters = new ArrayList<Parameter>();
			Iterator<Map.Entry<String, Object>> iterator = headers.entrySet().iterator();
			while (iterator.hasNext()){
				Map.Entry<String, Object> header = iterator.next();
				if (header.getValue() instanceof java.lang.String)
					parameters.add(new Parameter(header.getKey(), header.getValue().toString()));
				}
			/*for (Map.Entry<String, Object> header : headers.entrySet()){
				parameters.add(new Parameter(header.getKey(), header.getValue().toString()));
			}*/
			log.setParameters(parameters);

		}
				
		return log;
	}


}
