package au.com.nbnco.framework;

import java.sql.Connection;
import oracle.jdbc.OracleConnection;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import java.sql.CallableStatement;
import java.sql.ResultSet;
import java.util.List;
import java.sql.Array;
import java.sql.Struct;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.ibatis.type.BaseTypeHandler;
import org.apache.ibatis.type.JdbcType;

/**
 * Custom MyBatis Handler for the name value pairs that accompany every log/exception record.
 */

public class ParameterTypeHandler extends BaseTypeHandler<List<Parameter>> {
	
	 private final Log log = LogFactory.getLog(ParameterTypeHandler.class);
 
	 @Override
	 public void setParameter(PreparedStatement ps, int argNum,	
			 List<Parameter> reqparam, JdbcType jdbcType) throws SQLException {

		 if (null != reqparam) {
			
			Connection conn = ps.getConnection();
			
			List<Parameter> nvParamList = (List<Parameter>) reqparam;
			
			//2 items in the object array,one each for name and value
			Object[] nvParam = new Object[2];
			Struct[] nvParamStruct = new Struct[nvParamList.size()];
			
			//Create an array of java.sql.Struct objects
			//Each entry in this array will hold a name-value pair
			int index = 0;
			for (Parameter param: nvParamList){
				nvParam[0] = param.getName();
				nvParam[1] = param.getValue();				
				nvParamStruct[index++] = conn.createStruct("NVPARAM", nvParam);
			}
			
			//Create an instance of java.sql.Array to represent the table type. 
			//Use oracle.jdbc.OracleConnection.createARRAY instead of java.sql.Connection.createArrayOf
			//because Oracle does not support anonymous array types (all types are named). Refer to the link for more details
			//https://docs.oracle.com/cd/E11882_01/java.112/e16548/oraarr.htm#JJDBC28590
			Array arrayOfType = ((OracleConnection) conn).createARRAY("NVPARAMLIST", nvParamStruct);
			ps.setArray(argNum, arrayOfType);
		}
	 }
	 
	 
	 //The methods declared below need to be instantiated, because the class ParameterTypeHandler is not abstract
	 //Otherwise myBatis tries to instantiate the class and results in java.lang.InstantiationException
	 @Override
	  public void setNonNullParameter(PreparedStatement ps, int argNum,	
				 List<Parameter> reqparam, JdbcType jdbcType) throws SQLException {
		 //Do Nothing
	  }

	  @Override
	  public List<Parameter> getNullableResult(ResultSet resultSet, String columnName) throws SQLException {
		  return(null);
		  // Do Nothing
	  }

	  @Override
	  public List<Parameter> getNullableResult(ResultSet resultSet, int columnIndex) {
		  return(null);
		  // Do Nothing
	  }

	  @Override
	  public List<Parameter> getNullableResult(CallableStatement statement, int columnIndex) throws SQLException {
		  return(null);
		  // Do Nothing
	  }	 

}
